clear 
format long


%==========================================================================
%
% This program validates the linear approximation of the two dim manifold
% at the origin in lorenz.  The user sets the domain size nu and the values
% of sigma, beta, rho.
%
%
%==========================================================================


%Set the domain size.  Try to validate.
nu = intval(10^(-8))



%---------------------------Global Parameters------------------------------
%time scale
L = intval(1.0)



%Lorenz Parameters
sigma = intval('10')
beta = intval('8')/intval('3')
rho = intval('28')   %rho = 1.33 good starting point
           

%==========================================================================
%==========================Invariant Manifolds=============================
%==========================================================================

good = [0 0];

%-----------First order data at the origin--------------------------------


%Compute and assign names to linear data:
p0 = intval([0 0 0]);

Dfp0 = lorenzDifferential_intval(p0, sigma, rho, beta, L);

[U_temp, Sigma_temp] = eigs(Dfp0.mid)      

[lambda1_temp, xi1_temp] = ...
     verifyeig(Dfp0, Sigma_temp(1,1), U_temp(:,1))

[lambda2_temp, xi2_temp] = ...
     verifyeig(Dfp0, Sigma_temp(2,2), U_temp(:,2))

[lambda3_temp, xi3_temp] = ...
     verifyeig(Dfp0, Sigma_temp(3,3), U_temp(:,3))

 
xi1_temp
xi2_temp
xi3_temp
 
Q0 = [xi1_temp, xi2_temp, xi3_temp]
Q0_inv = inv(Q0)

lambda0_s1 = lambda1_temp
lambda0_s2 = lambda3_temp

xi0_s1 = xi1_temp
xi0_s2 = xi3_temp

lambda0_u = lambda2_temp
xi0_u = xi2_temp

A = [xi0_s1, xi0_s2]


%==========================================================================
%Validation Algorithm:
%==========================================================================

%STEP 1: ------------------------------------------------------------------
% Pick the size of the neighborhood to look at
R = intval(1)      %intval('2')/intval('1')      %In the notes this is rho instead of R
                                 %I'm using R here as rho is a system 
                                 %parameter for Lorenz


%STEP 2: ------------------------------------------------------------------
%Compute a bound on the size of the vector field in an R neighborhood
%of the equilibria:

M = intval(1)
N_f = intval(4)

%STEP 3: ------------------------------------------------------------------
%Compute the matrix norm bounds:
a = norm(A, inf)
Q1 = norm(Q0, inf)
Q2 = norm(Q0_inv, inf)

%STEP 4: ------------------------------------------------------------------

%Find the resonance bound N_0: How many conditions to check by hand in order
%to rule out resonances?

mu_subStar = abs(lambda0_s2)
mu_supStar = abs(lambda0_s1)

checkNum = inf(mu_subStar) - sup(mu_supStar); 
N_0 = 1;
while checkNum <= 0
    N_0 = N_0 + 1;
   checkNum = N_0*inf(mu_subStar) - sup(mu_supStar);
end

%Answer:
N_0

%Now find the bound on the nearest resonance in order to bound the 
%linear operator L.

maxRes = 1/(N_0*mu_subStar - mu_supStar)

%First for lambda0_s1:
for n = 2:N_0
    for k = 0:n
        ThisResCheck = intval(1)/abs((n-k)*lambda0_s1 + k*lambda0_s2 - lambda0_s1);
        if sup(maxRes) <= inf(ThisResCheck)
           maxRes = ThisResCheck;
           maxIndex = [n-k, k, lambda0_s1];
        end
    end
end


%Now check for lambda0_s2:
for n = 2:N_0
    for k = 0:n
        ThisResCheck = intval(1)/abs((n-k)*lambda0_s1 + k*lambda0_s2 - lambda0_s2);
        if sup(maxRes) <= inf(ThisResCheck)
           maxRes = ThisResCheck;
           maxIndex = [n-k, k, lambda0_s2];
        end
    end
end


maxRes

%STEP 5: ------------------------------------------------------------------
%    Define the bound on the linear operator:
C_tilde = Q1*Q2*maxRes


%STEP 6: ------------------------------------------------------------------
%Choose s and compute the combinatorial constants:

s = intval(10^(-2))
dim = 3     %Dimension of the phase space
C_n = linearApproxCombConst_C(s, dim)
K_n = linearApproxCombConst_K(s, dim)



%Step 7: ------------------------------------------------------------------
%Set nu:  Done Above


%nu = sqrt(epsilon_tol)


%Step 8: ------------------------------------------------------------------
%find delta_min and check non-degenerate:

B = C_tilde*N_f*M*C_n




if inf((2*a*B*nu - 1)^2 - 4*B^2*a^2*nu^2) <= 0
   failCondition = negativeDiscriminant 
end

if sup(2*a*B*nu) >= 1
   failCondition = b_not_negative 
end

%Use the quadratic formula to find the min and max delta:
delta_min  = (-(2*a*B*nu - 1) - ...
    sqrt((2*a*B*nu - 1)^2 - 4*B*a^2*B*nu^2))/(2*B) 


delta_max  = (-(2*a*B*nu - 1) + ...
    sqrt((2*a*B*nu - 1)^2 - 4*B*a^2*B*nu^2))/(2*B) 




%STEP 9: ------------------------------------------------------------------
%Check that s was small enough


step_9_number = (a*nu + delta_min)/R
if sup((a*nu + delta_min)/R) >= s
   failCondition = s_not_small_enough 
end

%STEP 10: -----------------------------------------------------------------

step_10_number = C_tilde*N_f*M*K_n*(a*nu + delta_min)

if sup(step_10_number) >= 1
   failCondition = lost_Z_bound 
end


%STEP 11: -----------------------------------------------------------------
%Return the values: if we made it here then:

%validated domain radius:
nu_is = sup(nu) 
%validated error bound:
delta_is = sup(delta_min)





